<?php

// This is a IA (anthropic) powered conversational Bot. You must supply your 
// Claude API key for it to work. You will also need to modify the SYSTEM_PROMPT
// with data about your company/service/product. This will use the token limit 
// of Claude, so you cannot have a very large prompt. Usually it is limited to
// 4096 tokens counting both the supplied prompt, the history context and the answer.
//
// For larger context we offer a paid bot option that uses RAG to expand knowledge
// almost without limits. Contact us for more info.

$DEBUG             = 1;
$ANTHROPIC_TOKEN   = '';
$ANTHROPIC_VERSION = '';
$MODEL             = "claude-3-haiku-20240307";

//claude-3-sonnet-20240229
//claude-3-opus-20240229

$GUARDRAILS_USER_PROMPT='. Do not justify your answer.';

$SYSTEM_PROMPT = "You are a helpful virtual assistant.";

$ACTION_PROMPT = "If the user is frustrated or angry, or if the user asks to connect to a real person, you will pass the conversation to a human by appending '[transfer]'. In the farewell/goodbye message you will append '[close]'.";

// override all hardcoded options if we have a bot name and ini file
if(isset($_REQUEST['bot'])) {
    $filename = getcwd()."/".$_REQUEST['bot'].".ini";
    if(is_readable($filename)) {
        $ini = parse_ini_file($filename,false);
        $ANTHROPIC_TOKEN = $ini['anthropic_token'];
        $ANTHROPIC_VERSION = $ini['anthropic_version'];
        $SYSTEM_PROMPT = $ini['system_prompt'];
        $ACTION_PROMPT = $ini['action_prompt'];
        $MAX_TOKENS = $ini['max_tokens'];
        $GUARDRAILS_USER_PROMPT = $ini['guardrails_prompt'];
        $MODEL = $ini['model'];
    }
}

$json = file_get_contents('php://input');
$data = json_decode($json,1);

log_debug("POST RECEIVED:\n".print_r($data,1));

$user_prompt = $data['body'];

$user_prompt.=$GUARDRAILS_USER_PROMPT; // add guardrails to question

$chat_context = array_slice($data['chat_context'],-6);

if (count($chat_context) > 0 && $chat_context[0]['role'] === 'assistant') {
    array_shift($chat_context);
}

$chat_context[] = array("role"=>"user","content"=>$user_prompt);

if($ANTHROPIC_TOKEN=='') {
    $response = "Bot is not yet configured, be sure to set up your Claude API key";
} else {
    $response = ask_claude($ANTHROPIC_TOKEN,$SYSTEM_PROMPT." ".$ACTION_PROMPT,$chat_context);
}

$return = array("body"=>$response,"attach"=>'');
log_debug("REPLY: $response");
echo json_encode($return);
die();

function ask_claude($anthropic_token,$system_prompt,$user_prompt) {
    global $MODEL, $MAX_TOKENS, $ANTHROPIC_VERSION;

    $url = 'https://api.anthropic.com/v1/messages';
    $data = array(
        "model"=> $MODEL,
        "max_tokens"=> intval($MAX_TOKENS),
        "messages" => $user_prompt,
        "system" => $system_prompt,
    );
    $data_string = json_encode($data);
    log_debug("Claude Data: ".print_r($data,1));
    log_debug("Claude Messages: ".print_r($user_prompt,1));
    $headers=array('Content-Type: application/json','Content-Length: ' . strlen($data_string),"x-api-key: $anthropic_token","anthropic-version: $ANTHROPIC_VERSION");
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_TIMEOUT, 120);
    curl_setopt($ch, CURLOPT_VERBOSE, true);
    curl_setopt($ch, CURLOPT_SSLVERSION, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $return = curl_exec($ch);
    curl_close($ch);
    $response = json_decode($return,1);
    log_debug("Claude response: ".print_r($response,1));
    return $response['content'][0]['text'];
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}
